@@ -25,7 +25,8 @@
 #include <gst/gst.h>
 #include <gst/base/gstcollectpads.h>
 
-#include <ogg/ogg.h>
+#include "gstoggmux.h"
+
 /* memcpy - if someone knows a way to get rid of it, please speak up
  * note: the ogg docs even say you need this... */
 #include <string.h>
@@ -35,12 +36,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
 #define GST_CAT_DEFAULT gst_ogg_mux_debug
 
-#define GST_TYPE_OGG_MUX (gst_ogg_mux_get_type())
-#define GST_OGG_MUX(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_OGG_MUX, GstOggMux))
-#define GST_OGG_MUX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_OGG_MUX, GstOggMux))
-#define GST_IS_OGG_MUX(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_OGG_MUX))
-#define GST_IS_OGG_MUX_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OGG_MUX))
-
 /* This isn't generally what you'd want with an end-time macro, because
    technically the end time of a buffer with invalid duration is invalid. But
    for sorting ogg pages this is what we want. */
@@ -51,81 +46,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
 
 #define GST_GP_FORMAT "[gp %8" G_GINT64_FORMAT "]"
 
-typedef struct _GstOggMux GstOggMux;
-typedef struct _GstOggMuxClass GstOggMuxClass;
-
-typedef enum
-{
-  GST_OGG_PAD_STATE_CONTROL = 0,
-  GST_OGG_PAD_STATE_DATA = 1
-}
-GstOggPadState;
-
-/* all information needed for one ogg stream */
-typedef struct
-{
-  GstCollectData collect;       /* we extend the CollectData */
-
-  GstBuffer *buffer;            /* the queued buffer for this pad */
-
-  gint serial;
-  ogg_stream_state stream;
-  gint64 packetno;              /* number of next packet */
-  gint64 pageno;                /* number of next page */
-  guint64 duration;             /* duration of current page */
-  gboolean eos;
-  gint64 offset;
-  GstClockTime timestamp;       /* timestamp of the first packet on the next
-                                 * page to be dequeued */
-  GstClockTime timestamp_end;   /* end timestamp of last complete packet on
-                                   the next page to be dequeued */
-  GstClockTime gp_time;         /* time corresponding to the gp value of the
-                                   last complete packet on the next page to be
-                                   dequeued */
-
-  GstOggPadState state;         /* state of the pad */
-
-  GList *headers;
-
-  GQueue *pagebuffers;          /* List of pages in buffers ready for pushing */
-
-  gboolean new_page;            /* starting a new page */
-  gboolean first_delta;         /* was the first packet in the page a delta */
-  gboolean prev_delta;          /* was the previous buffer a delta frame */
-}
-GstOggPad;
-
-struct _GstOggMux
-{
-  GstElement element;
-
-  /* source pad */
-  GstPad *srcpad;
-
-  /* sinkpads */
-  GstCollectPads *collect;
-
-  /* the pad we are currently using to fill a page */
-  GstOggPad *pulling;
-
-  /* next timestamp for the page */
-  GstClockTime next_ts;
-
-  /* offset in stream */
-  guint64 offset;
-
-  /* need_headers */
-  gboolean need_headers;
-
-  guint64 max_delay;
-  guint64 max_page_delay;
-
-  GstOggPad *delta_pad;         /* when a delta frame is detected on a stream, we mark
-                                   pages as delta frames up to the page that has the
-                                   keyframe */
-
-};
-
 typedef enum
 {
   GST_OGG_FLAG_BOS = GST_ELEMENT_FLAG_LAST,
@@ -133,11 +53,6 @@ typedef enum
 }
 GstOggFlag;
 
-struct _GstOggMuxClass
-{
-  GstElementClass parent_class;
-};
-
 /* elementfactory information */
 static const GstElementDetails gst_ogg_mux_details =
 GST_ELEMENT_DETAILS ("Ogg muxer",
@@ -293,6 +208,7 @@ gst_ogg_mux_clear (GstOggMux * ogg_mux)
   ogg_mux->delta_pad = NULL;
   ogg_mux->offset = 0;
   ogg_mux->next_ts = 0;
+  ogg_mux->last_ts = GST_CLOCK_TIME_NONE;
 }
 
 static void
@@ -334,6 +250,23 @@ gst_ogg_mux_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gst_ogg_mux_ogg_pad_destroy_notify (GstCollectData * data)
+{
+  GstOggPad *oggpad = (GstOggPad *) data;
+  GstBuffer *buf;
+
+  ogg_stream_clear (&oggpad->stream);
+
+  if (oggpad->pagebuffers) {
+    while ((buf = g_queue_pop_head (oggpad->pagebuffers)) != NULL) {
+      gst_buffer_unref (buf);
+    }
+    g_queue_free (oggpad->pagebuffers);
+    oggpad->pagebuffers = NULL;
+  }
+}
+
 static GstPadLinkReturn
 gst_ogg_mux_sinkconnect (GstPad * pad, GstPad * peer)
 {
@@ -381,6 +314,7 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       serial = atoi (&req_name[5]);
     }
     /* create new pad with the name */
+    GST_DEBUG_OBJECT (ogg_mux, "Creating new pad for serial %d", serial);
     name = g_strdup_printf ("sink_%d", serial);
     newpad = gst_pad_new_from_template (templ, name);
     g_free (name);
@@ -391,8 +325,9 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       GstOggPad *oggpad;
 
       oggpad = (GstOggPad *)
-          gst_collect_pads_add_pad (ogg_mux->collect, newpad,
-          sizeof (GstOggPad));
+          gst_collect_pads_add_pad_full (ogg_mux->collect, newpad,
+          sizeof (GstOggPad), gst_ogg_mux_ogg_pad_destroy_notify);
+      ogg_mux->active_pads++;
 
       oggpad->serial = serial;
       ogg_stream_init (&oggpad->stream, serial);
@@ -432,36 +367,13 @@ static void
 gst_ogg_mux_release_pad (GstElement * element, GstPad * pad)
 {
   GstOggMux *ogg_mux;
-  GSList *walk;
 
   ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
 
-  /* FIXME: When a request pad is released while paused or playing, 
-   * we probably need to do something to finalise its stream in the
-   * ogg data we're producing, but I'm not sure what */
-
-  /* Find out GstOggPad in the collect pads info and clean it up */
-
-  GST_OBJECT_LOCK (ogg_mux->collect);
-  for (walk = ogg_mux->collect->data; walk; walk = g_slist_next (walk)) {
-    GstOggPad *oggpad = (GstOggPad *) walk->data;
-    GstCollectData *cdata = (GstCollectData *) walk->data;
-    GstBuffer *buf;
-
-    if (cdata->pad == pad) {
-      ogg_stream_clear (&oggpad->stream);
-
-      while ((buf = g_queue_pop_head (oggpad->pagebuffers)) != NULL) {
-        gst_buffer_unref (buf);
-      }
-
-      g_queue_free (oggpad->pagebuffers);
-      oggpad->pagebuffers = NULL;
-    }
-  }
-  GST_OBJECT_UNLOCK (ogg_mux->collect);
-
   gst_collect_pads_remove_pad (ogg_mux->collect, pad);
+  gst_element_remove_pad (element, pad);
+
+  gst_object_unref (ogg_mux);
 }
 
 /* handle events */
@@ -517,6 +429,15 @@ gst_ogg_mux_push_buffer (GstOggMux * mux, GstBuffer * buffer)
   mux->offset += GST_BUFFER_SIZE (buffer);
   GST_BUFFER_OFFSET_END (buffer) = mux->offset;
 
+  /* Ensure we have monotonically increasing timestamps in the output. */
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer)) {
+    if (mux->last_ts != GST_CLOCK_TIME_NONE &&
+        GST_BUFFER_TIMESTAMP (buffer) < mux->last_ts)
+      GST_BUFFER_TIMESTAMP (buffer) = mux->last_ts;
+    else
+      mux->last_ts = GST_BUFFER_TIMESTAMP (buffer);
+  }
+
   caps = gst_pad_get_negotiated_caps (mux->srcpad);
   gst_buffer_set_caps (buffer, caps);
   gst_caps_unref (caps);
@@ -692,18 +613,24 @@ gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
 
   /* if the first pad doesn't contain anything or is even NULL, return
    * the second pad as best candidate and vice versa */
-  if (first == NULL || first->buffer == NULL)
+  if (first == NULL || (first->buffer == NULL && first->next_buffer == NULL))
     return 1;
-  if (second == NULL || second->buffer == NULL)
+  if (second == NULL || (second->buffer == NULL && second->next_buffer == NULL))
     return -1;
 
   /* no timestamp on first buffer, it must go first */
-  firsttime = GST_BUFFER_TIMESTAMP (first->buffer);
+  if (first->buffer)
+    firsttime = GST_BUFFER_TIMESTAMP (first->buffer);
+  else
+    firsttime = GST_BUFFER_TIMESTAMP (first->next_buffer);
   if (firsttime == GST_CLOCK_TIME_NONE)
     return -1;
 
   /* no timestamp on second buffer, it must go first */
-  secondtime = GST_BUFFER_TIMESTAMP (second->buffer);
+  if (second->buffer)
+    secondtime = GST_BUFFER_TIMESTAMP (second->buffer);
+  else
+    secondtime = GST_BUFFER_TIMESTAMP (second->next_buffer);
   if (secondtime == GST_CLOCK_TIME_NONE)
     return 1;
 
@@ -726,9 +653,19 @@ gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
   return 0;
 }
 
-/* make sure a buffer is queued on all pads, returns a pointer to an oggpad
- * that holds the best buffer or NULL when no pad was usable.
- * "best" means the buffer marked with the lowest timestamp */
+/* make sure at least one buffer is queued on all pads, two if possible
+ * 
+ * if pad->buffer == NULL, pad->next_buffer !=  NULL, then
+ *   we do not know if the buffer is the last or not
+ * if pad->buffer != NULL, pad->next_buffer != NULL, then
+ *   pad->buffer is not the last buffer for the pad
+ * if pad->buffer != NULL, pad->next_buffer == NULL, then
+ *   pad->buffer if the last buffer for the pad
+ * 
+ * returns a pointer to an oggpad that holds the best buffer, or
+ * NULL when no pad was usable. "best" means the buffer marked
+ * with the lowest timestamp. If best->buffer == NULL then nothing
+ * should be done until more data arrives */
 static GstOggPad *
 gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 {
@@ -753,11 +690,23 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
       GstBuffer *buf;
       gboolean incaps;
 
+      /* shift the buffer along if needed (it's okay if next_buffer is NULL) */
+      if (pad->buffer == NULL) {
+        GST_LOG_OBJECT (data->pad, "shifting buffer %" GST_PTR_FORMAT,
+            pad->next_buffer);
+        pad->buffer = pad->next_buffer;
+        pad->next_buffer = NULL;
+      }
+
       buf = gst_collect_pads_pop (ogg_mux->collect, data);
       GST_LOG_OBJECT (data->pad, "popped buffer %" GST_PTR_FORMAT, buf);
 
       /* On EOS we get a NULL buffer */
       if (buf != NULL) {
+        if (ogg_mux->delta_pad == NULL &&
+            GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT))
+          ogg_mux->delta_pad = pad;
+
         incaps = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
         /* if we need headers */
         if (pad->state == GST_OGG_PAD_STATE_CONTROL) {
@@ -781,6 +730,9 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
           ogg_page page;
           GstFlowReturn ret;
 
+          /* it's no longer active */
+          ogg_mux->active_pads--;
+
           /* Just gone to EOS. Flush existing page(s) */
           pad->eos = TRUE;
 
@@ -796,14 +748,16 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
         }
       }
 
-      pad->buffer = buf;
+      pad->next_buffer = buf;
     }
 
     /* we should have a buffer now, see if it is the best pad to
      * pull on */
-    if (pad->buffer) {
+    if (pad->buffer || pad->next_buffer) {
       if (gst_ogg_mux_compare_pads (ogg_mux, bestpad, pad) > 0) {
-        GST_LOG_OBJECT (data->pad, "new best pad");
+        GST_LOG_OBJECT (data->pad,
+            "new best pad, with buffers %" GST_PTR_FORMAT
+            " and %" GST_PTR_FORMAT, pad->buffer, pad->next_buffer);
 
         bestpad = pad;
       }
@@ -840,33 +794,49 @@ gst_ogg_mux_get_headers (GstOggPad * pad)
     const GValue *streamheader;
 
     structure = gst_caps_get_structure (caps, 0);
-    streamheader = gst_structure_get_value (structure, "streamheader");
-    if (streamheader != NULL) {
-      GST_LOG_OBJECT (thepad, "got header");
-      if (G_VALUE_TYPE (streamheader) == GST_TYPE_ARRAY) {
-        GArray *bufarr = g_value_peek_pointer (streamheader);
-        gint i;
+    if (strcmp (gst_structure_get_name (structure), "video/x-dirac") == 0) {
+      GstBuffer *buf = gst_buffer_new_and_alloc (16);
+      int fps_n = 12;
+      int fps_d = 1;
 
-        GST_LOG_OBJECT (thepad, "got fixed list");
+      gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d);
 
-        for (i = 0; i < bufarr->len; i++) {
-          GValue *bufval = &g_array_index (bufarr, GValue, i);
+      memcpy (GST_BUFFER_DATA (buf), "KW-DIRAC", 8);
+      GST_WRITE_UINT32_BE (GST_BUFFER_DATA (buf) + 8, fps_n);
+      GST_WRITE_UINT32_BE (GST_BUFFER_DATA (buf) + 12, fps_d);
 
-          GST_LOG_OBJECT (thepad, "item %d", i);
-          if (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER) {
-            GstBuffer *buf = g_value_peek_pointer (bufval);
+      res = g_list_append (res, buf);
 
-            GST_LOG_OBJECT (thepad, "adding item %d to header list", i);
+      //res = g_list_append (res, gst_buffer_ref(pad->buffer));
+    } else {
+      streamheader = gst_structure_get_value (structure, "streamheader");
+      if (streamheader != NULL) {
+        GST_LOG_OBJECT (thepad, "got header");
+        if (G_VALUE_TYPE (streamheader) == GST_TYPE_ARRAY) {
+          GArray *bufarr = g_value_peek_pointer (streamheader);
+          gint i;
 
-            gst_buffer_ref (buf);
-            res = g_list_append (res, buf);
+          GST_LOG_OBJECT (thepad, "got fixed list");
+
+          for (i = 0; i < bufarr->len; i++) {
+            GValue *bufval = &g_array_index (bufarr, GValue, i);
+
+            GST_LOG_OBJECT (thepad, "item %d", i);
+            if (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER) {
+              GstBuffer *buf = g_value_peek_pointer (bufval);
+
+              GST_LOG_OBJECT (thepad, "adding item %d to header list", i);
+
+              gst_buffer_ref (buf);
+              res = g_list_append (res, buf);
+            }
           }
+        } else {
+          GST_LOG_OBJECT (thepad, "streamheader is not fixed list");
         }
       } else {
-        GST_LOG_OBJECT (thepad, "streamheader is not fixed list");
+        GST_LOG_OBJECT (thepad, "caps don't have streamheader");
       }
-    } else {
-      GST_LOG_OBJECT (thepad, "caps done have streamheader");
     }
     gst_caps_unref (caps);
   } else {
@@ -947,7 +917,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GST_LOG_OBJECT (mux, "looking at pad %s:%s", GST_DEBUG_PAD_NAME (thepad));
 
     /* if the pad has no buffer, we don't care */
-    if (pad->buffer == NULL)
+    if (pad->buffer == NULL && pad->next_buffer == NULL)
       continue;
 
     /* now figure out the headers */
@@ -983,6 +953,9 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     } else if (pad->buffer) {
       buf = pad->buffer;
       gst_buffer_ref (buf);
+    } else if (pad->next_buffer) {
+      buf = pad->next_buffer;
+      gst_buffer_ref (buf);
     } else {
       /* fixme -- should be caught in the previous list traversal. */
       GST_OBJECT_LOCK (pad);
@@ -1017,11 +990,17 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GST_LOG_OBJECT (mux, "swapped out page with mime type %s",
         gst_structure_get_name (structure));
 
-    /* quick hack: put theora pages at the front.
+    /* quick hack: put Theora and Dirac video pages at the front.
      * Ideally, we would have a settable enum for which Ogg
-     * profile we work with, and order based on that */
-    if (strcmp (gst_structure_get_name (structure), "video/x-theora") == 0) {
-      GST_DEBUG_OBJECT (thepad, "putting Theora page at the front");
+     * profile we work with, and order based on that.
+     * (FIXME: if there is more than one video stream, shouldn't we only put
+     * one's BOS into the first page, followed by an audio stream's BOS, and
+     * only then followed by the remaining video and audio streams?) */
+    if (gst_structure_has_name (structure, "video/x-theora")) {
+      GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Theora");
+      hbufs = g_list_prepend (hbufs, hbuf);
+    } else if (gst_structure_has_name (structure, "video/x-dirac")) {
+      GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Dirac");
       hbufs = g_list_prepend (hbufs, hbuf);
     } else {
       hbufs = g_list_append (hbufs, hbuf);
@@ -1070,8 +1049,8 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
       /* if last header, flush page */
       if (hwalk == NULL) {
         GST_LOG_OBJECT (mux,
-            "flushing page as packet %d is first or last packet",
-            pad->packetno);
+            "flushing page as packet %" G_GUINT64_FORMAT " is first or "
+            "last packet", pad->packetno);
         while (ogg_stream_flush (&pad->stream, &page)) {
           GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
@@ -1116,59 +1095,44 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
   return ret;
 }
 
-/* this function is called when there is data on all pads.
+/* this function is called to process data on the best pending pad.
  *
  * basic idea:
  *
- * 1) find a pad to pull on, this is done by looking at the buffers
- *    to decide which one to use, we use the 'oldest' one first.
- * 2) store the selected pad and keep on pulling until we fill a
+ * 1) store the selected pad and keep on pulling until we fill a
  *    complete ogg page or the ogg page is filled above the max-delay
  *    threshold. This is needed because the ogg spec says that
  *    you should fill a complete page with data from the same logical
  *    stream. When the page is filled, go back to 1).
- * 3) before filling a page, read ahead one more buffer to see if this
+ * 2) before filling a page, read ahead one more buffer to see if this
  *    packet is the last of the stream. We need to do this because the ogg
  *    spec mandates that the last packet should have the EOS flag set before
- *    sending it to ogg. FIXME: Apparently we're allowed to send empty 'nil'
- *    pages with the EOS flag set for EOS, so we could do this. Not sure how
- *    that works, though. TODO: 'read ahead one more buffer' is a bit funky
- *    with collectpads. Rethink this.
- * 4) pages get queued on a per-pad queue. Every time a page is queued, a
+ *    sending it to ogg. if pad->buffer is NULL we need to wait to find out
+ *    whether there are any more buffers.
+ * 3) pages get queued on a per-pad queue. Every time a page is queued, a
  *    dequeue is called, which will dequeue the oldest page on any pad, provided
  *    that ALL pads have at least one marked page in the queue (or remaining
- *    pad are at EOS)
+ *    pads are at EOS)
  */
 static GstFlowReturn
-gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
+gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
 {
-  GstOggPad *best;
   gboolean delta_unit;
   GstFlowReturn ret;
   gint64 granulepos = 0;
   GstClockTime timestamp, gp_time;
 
-  GST_LOG_OBJECT (ogg_mux, "collected");
-
-  /* queue buffers on all pads; find a buffer with the lowest timestamp */
-  best = gst_ogg_mux_queue_pads (ogg_mux);
-  if (best && !best->buffer) {
-    GST_DEBUG_OBJECT (ogg_mux, "No buffer available on best pad");
-    return GST_FLOW_OK;
-  }
-
-  if (!best) {
-    /* EOS : FIXME !! We need to handle EOS correctly, and set EOS
-       flags on the ogg pages. */
-    GST_DEBUG_OBJECT (ogg_mux, "Pushing EOS");
-    gst_pad_push_event (ogg_mux->srcpad, gst_event_new_eos ());
-    return GST_FLOW_WRONG_STATE;
-  }
-
   GST_LOG_OBJECT (ogg_mux, "best pad %" GST_PTR_FORMAT
       ", currently pulling from %" GST_PTR_FORMAT, best->collect.pad,
       ogg_mux->pulling);
 
+  /* best->buffer is non-NULL, either the pad is EOS's or there is a next 
+   * buffer */
+  if (best->next_buffer == NULL && !best->eos) {
+    GST_WARNING_OBJECT (ogg_mux, "no subsequent buffer and EOS not reached");
+    return GST_FLOW_WRONG_STATE;
+  }
+
   /* if we were already pulling from one pad, but the new "best" buffer is
    * from another pad, we need to check if we have reason to flush a page
    * for the pad we were pulling from before */
@@ -1270,7 +1234,7 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
         " packet %" G_GINT64_FORMAT " (%ld bytes) created from buffer",
         packet.granulepos, packet.packetno, packet.bytes);
 
-    packet.e_o_s = 0;
+    packet.e_o_s = (pad->eos ? 1 : 0);
     tmpbuf = NULL;
 
     /* we flush when we see a new keyframe */
@@ -1284,6 +1248,13 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
       }
     }
 
+    if (GST_BUFFER_IS_DISCONT (buf)) {
+      packet.packetno++;
+      /* No public API for this; hack things in */
+      pad->stream.pageno++;
+      force_flush = TRUE;
+    }
+
     /* flush the currently built page if necessary */
     if (force_flush) {
       GST_LOG_OBJECT (pad->collect.pad,
@@ -1306,11 +1277,6 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     /* if this is the first packet of a new page figure out the delta flag */
     if (pad->new_page) {
       if (delta_unit) {
-        /* This page is a delta frame */
-        if (ogg_mux->delta_pad == NULL) {
-          /* we got a delta unit on this pad */
-          ogg_mux->delta_pad = pad;
-        }
         /* mark the page as delta */
         pad->first_delta = TRUE;
       } else {
@@ -1367,7 +1333,7 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
           GST_TIME_FORMAT ") caused new page",
           granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
       GST_LOG_OBJECT (pad->collect.pad,
-          GST_GP_FORMAT " new page %d", ogg_page_granulepos (&page),
+          GST_GP_FORMAT " new page %ld", ogg_page_granulepos (&page),
           pad->stream.pageno);
 
       if (ogg_page_granulepos (&page) == granulepos) {
@@ -1434,6 +1400,94 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
   return GST_FLOW_OK;
 }
 
+/** all_pads_eos:
+ *
+ * Checks if all pads are EOS'd by peeking.
+ *
+ * Returns TRUE if all pads are EOS.
+ */
+static gboolean
+all_pads_eos (GstCollectPads * pads)
+{
+  GSList *walk;
+  gboolean alleos = TRUE;
+
+  walk = pads->data;
+  while (walk) {
+    GstBuffer *buf;
+    GstCollectData *data = (GstCollectData *) walk->data;
+
+    buf = gst_collect_pads_peek (pads, data);
+    if (buf) {
+      alleos = FALSE;
+      gst_buffer_unref (buf);
+      goto beach;
+    }
+    walk = walk->next;
+  }
+beach:
+  return alleos;
+}
+
+/* This function is called when there is data on all pads.
+ * 
+ * It finds a pad to pull on, this is done by looking at the buffers
+ * to decide which one to use, and using the 'oldest' one first. It then calls
+ * gst_ogg_mux_process_best_pad() to process as much data as possible.
+ * 
+ * If all the pads have received EOS, it flushes out all data by continually
+ * getting the best pad and calling gst_ogg_mux_process_best_pad() until they
+ * are all empty, and then sends EOS.
+ */
+static GstFlowReturn
+gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
+{
+  GstOggPad *best;
+  GstFlowReturn ret;
+  gint activebefore;
+
+  GST_LOG_OBJECT (ogg_mux, "collected");
+
+  activebefore = ogg_mux->active_pads;
+
+  /* queue buffers on all pads; find a buffer with the lowest timestamp */
+  best = gst_ogg_mux_queue_pads (ogg_mux);
+  if (best && !best->buffer) {
+    GST_DEBUG_OBJECT (ogg_mux, "No buffer available on best pad");
+    return GST_FLOW_OK;
+  }
+
+  if (!best) {
+    return GST_FLOW_WRONG_STATE;
+  }
+
+  ret = gst_ogg_mux_process_best_pad (ogg_mux, best);
+
+  if (ogg_mux->active_pads < activebefore) {
+    /* If the active pad count went down, this mean at least one pad has gone
+     * EOS. Since CollectPads only calls _collected() once when all pads are
+     * EOS, and our code doesn't _pop() from all pads we need to check that by
+     * peeking on all pads, else we won't be called again and the muxing will
+     * not terminate (push out EOS). */
+
+    /* if all the pads have been removed, flush all pending data */
+    if ((ret == GST_FLOW_OK) && all_pads_eos (pads)) {
+      GST_LOG_OBJECT (ogg_mux, "no pads remaining, flushing data");
+
+      do {
+        best = gst_ogg_mux_queue_pads (ogg_mux);
+        if (best)
+          ret = gst_ogg_mux_process_best_pad (ogg_mux, best);
+      } while ((ret == GST_FLOW_OK) && (best != NULL));
+
+      GST_DEBUG_OBJECT (ogg_mux, "Pushing EOS");
+      gst_pad_push_event (ogg_mux->srcpad, gst_event_new_eos ());
+    }
+  }
+
+  return ret;
+}
+
 static void
 gst_ogg_mux_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
