@@ -176,6 +176,9 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_UYVY422:
       fmt = GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
       break;
+    case PIX_FMT_UYVY411:
+      fmt = GST_MAKE_FOURCC ('I', 'Y', 'U', '1');
+      break;
     case PIX_FMT_RGB24:
       bpp = depth = 24;
       endianness = G_BIG_ENDIAN;
@@ -224,19 +227,47 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       b_mask = 0x000000ff;
 #endif
       break;
-    case PIX_FMT_RGBA32:
+    case PIX_FMT_xRGB32:
       bpp = 32;
-      depth = 32;
+      depth = 24;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+#else
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+#endif
+      break;
+    case PIX_FMT_BGRx32:
+      bpp = 32;
+      depth = 24;
       endianness = G_BIG_ENDIAN;
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
       r_mask = 0x000000ff;
       g_mask = 0x0000ff00;
       b_mask = 0x00ff0000;
-      a_mask = 0xff000000;
 #else
       r_mask = 0xff000000;
       g_mask = 0x00ff0000;
       b_mask = 0x0000ff00;
+#endif
+      break;
+    case PIX_FMT_RGBA32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x00ff0000;
+      g_mask = 0x0000ff00;
+      b_mask = 0x000000ff;
+      a_mask = 0xff000000;
+#else
+      r_mask = 0x0000ff00;
+      g_mask = 0x00ff0000;
+      b_mask = 0xff000000;
       a_mask = 0x000000ff;
 #endif
       break;
@@ -254,7 +285,38 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       g_mask = 0x0000ff00;
       b_mask = 0x000000ff;
       a_mask = 0xff000000;
-
+#endif
+      break;
+    case PIX_FMT_ARGB32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+      a_mask = 0x000000ff;
+#else
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+      a_mask = 0xff000000;
+#endif
+      break;
+    case PIX_FMT_ABGR32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+      a_mask = 0xff000000;
+#else
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+      a_mask = 0x000000ff;
 #endif
       break;
     case PIX_FMT_YUV410P:
@@ -473,7 +535,7 @@ gst_ffmpeg_caps_to_smpfmt (const GstCaps * caps,
 
   if (gst_structure_get_int (structure, "width", &width) &&
       gst_structure_get_int (structure, "depth", &depth) &&
-      gst_structure_get_int (structure, "signed", &signedness) &&
+      gst_structure_get_boolean (structure, "signed", &signedness) &&
       gst_structure_get_int (structure, "endianness", &endianness)) {
     if (width == 16 && depth == 16 &&
         endianness == G_BYTE_ORDER && signedness == TRUE) {
@@ -525,6 +587,9 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
           context->pix_fmt = PIX_FMT_UYVY422;
           break;
+        case GST_MAKE_FOURCC ('I', 'Y', 'U', '1'):
+          context->pix_fmt = PIX_FMT_UYVY411;
+          break;
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
           context->pix_fmt = PIX_FMT_YUV420P;
           break;
@@ -562,26 +627,43 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
             if (gst_structure_get_int (structure, "alpha_mask", &amask)) {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x0000ff00)
+                context->pix_fmt = PIX_FMT_BGRA32;
+              else if (rmask == 0x00ff0000)
+                context->pix_fmt = PIX_FMT_RGBA32;
+              else if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_ARGB32;
+              else              // if (r_mask = 0x000000ff)
+                context->pix_fmt = PIX_FMT_ABGR32;
 #else
               if (rmask == 0x00ff0000)
-#endif
                 context->pix_fmt = PIX_FMT_BGRA32;
-              else
+              else if (rmask == 0x0000ff00)
                 context->pix_fmt = PIX_FMT_RGBA32;
+              else if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_ARGB32;
+              else              // if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_ABGR32;
+#endif
             } else {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x00ff0000)
-#else
-              if (rmask == 0x0000ff00)
-#endif
                 context->pix_fmt = PIX_FMT_RGB32;
-
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-              if (rmask == 0x0000ff00)
-#else
-              if (rmask == 0x00ff0000)
-#endif
+              else if (rmask == 0x0000ff00)
                 context->pix_fmt = PIX_FMT_BGR32;
+              else if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_xRGB32;
+              else              // if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_BGRx32;
+#else
+              if (rmask == 0x0000ff00)
+                context->pix_fmt = PIX_FMT_RGB32;
+              else if (rmask == 0x00ff0000)
+                context->pix_fmt = PIX_FMT_BGR32;
+              else if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_xRGB32;
+              else              // if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_BGRx32;
+#endif
             }
             break;
           case 24:
@@ -729,8 +811,12 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
     case PIX_FMT_AYUV4444:
     case PIX_FMT_RGB32:
     case PIX_FMT_RGBA32:
+    case PIX_FMT_ARGB32:
     case PIX_FMT_BGR32:
     case PIX_FMT_BGRA32:
+    case PIX_FMT_ABGR32:
+    case PIX_FMT_xRGB32:
+    case PIX_FMT_BGRx32:
       stride = width * 4;
       size = stride * height;
       picture->data[0] = ptr;
